package mercury;

import mercury.util.*;
import mercury.exception.*;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Date;

public class Monitor implements Application {

	Config config = Config.getInstance();
	boolean debug = new Boolean(config.getProperty("debug")).booleanValue();
	int _PERIOD = Integer.parseInt(config.getProperty("xander.monitor.wait_interval")); // period in seconds
	int _THREADS = Integer.parseInt(config.getProperty("xander.monitor.no_of_threads"));
	
	static int _threadCount = 0;
	Date _start = new Date();
	

	public Monitor() {
		if (debug) System.out.println("Monitor created.");
	}
	
	public void activate() {
		
		Date now = new Date();
		
		long start_millis = _start.getTime();
		long now_millis = now.getTime();
		
		if (now_millis > (start_millis + (_PERIOD * 1000))) {
			_start = now;
			monitor();		
		}
		
	}
	
	private void monitor() {
		if (debug) System.out.println("Triggering the monitor now!");
		
		try {
			TaskRegistry registry = new TaskRegistry();
			Task [] tasks  = registry.getAllTasks();
			
			if (debug) System.out.println("No of tasks found:" + tasks.length);
			
			ArrayList taskList = new ArrayList();
			
			for (int i=0; i<tasks.length;i++) {
				taskList.add(tasks[i]);
			}				
			while (!taskList.isEmpty()) {
				ArrayList dupTaskList = (ArrayList)taskList.clone();
				for (Iterator itr = dupTaskList.iterator(); itr.hasNext();) {
					Task task = (Task)itr.next();
					if (debug) System.out.println("## Monitor found task:" + task.getID() + " in registry.");
					
					if (_threadCount < _THREADS) {
						TaskProcessingThread thread = new TaskProcessingThread(task);
						thread.start();
						taskList.remove(taskList.indexOf(task));
					}
					else {
						if (debug) System.out.println("## Not enough worker threads (" + _THREADS + ")! Waiting for free thread ...");
					}
				}
			}
			
			
		}
		catch (TaskNotFoundException e) {
			System.out.println("### No tasks found in the task registry.");	
			e.printStackTrace();
		}	
		catch (Exception e) {
			System.err.println("### General monitor error.");
			e.printStackTrace();
			System.exit(-1);
		}
	}

}

class TaskProcessingThread extends Thread {
	
	Task task;
	
	public TaskProcessingThread() {};
	
	public TaskProcessingThread(Task threadedTask) {
		Monitor._threadCount++;
		task = threadedTask;
	}
	
	public void run() {
		if (task != null && task.getAllConditions() != null) {
			
			System.out.println("Processing task:" + task.getID() + "...");				
			TaskProcessor processor = new TaskProcessor();
			processor.process(task);
	
			if (Monitor._threadCount > 0)
				Monitor._threadCount--;
		}
	}	
	
}


