package mercury;

import java.net.*;
import java.io.*;
import java.util.*;
import java.text.NumberFormat;
import java.text.DecimalFormat;


/**
 * <code>MultiServerThread</code> is an extended thread that is used to respond
 * to TCP requests.
 * 
 * last modified - 6 April 2001<br>
 * version 1.0<br>
 * author Chang Sau Sheong
 *
 */

public class MultiServerThread extends Thread {

	Status status = Status.getInstance();
      
	// instance variable	
	private Socket _socket = null;


	/**
	 constructor
	*/
	
 	public MultiServerThread(Socket socket) {
  	super("MultiServerThread");
   	this._socket = socket;
 		//System.out.println ("Accepted from " + socket.getInetAddress());
 	}


	/**
	 run() method
	*/

 	public void run() {
		try {
	    PrintWriter out = new PrintWriter(_socket.getOutputStream(), true);
	    BufferedReader in = new BufferedReader(new InputStreamReader(_socket.getInputStream()));
		
			String inputLine = new String();
							
	   	while ((inputLine = in.readLine()) != null) {	   				
				StringTokenizer st = new StringTokenizer(inputLine, " ");
				String command = st.nextToken();
				Vector parameters = new Vector();
				
				while (st.hasMoreTokens()) {
						parameters.add(st.nextElement());
				}
				
				
				// request for change
				if (command.equalsIgnoreCase("change")) {
			   	if (!parameters.isEmpty()) {
			   		//System.out.println("change " + parameters.get(0));
			   		if (doChange(parameters)) {
			   			out.println("*");
			   		}
			   		else {
			   			out.println("##Error: No changes made.");
			   		}
			   	}
			   	else {
			   		out.println("##Error: Try change <parameter>");
			   	}
				} // end request for change
				

				// request for information
				else if (command.equalsIgnoreCase("ask")) {
			   	if (!parameters.isEmpty()) {
			   		//System.out.println("ask " + parameters.get(0));
			   		out.println(doAsk(parameters));
			   	}
			   	else {
			   		out.println("##Error: Try ask <parameter>");
			   	}			   	
			 	}			 	
			 	else {
			 		out.println("##Error: No such command.");
			 	} // end request for information
			 	
	  	}	// end while loop
	  } 
	  catch (NumberFormatException nfe) {
			System.err.println("### Port passed is not a number." + nfe.getMessage());
		} 
		catch (java.net.SocketException se) {
			System.err.println("### Connection closed by client." + se.getMessage());
  	} 
  	catch (Exception oe) {
			System.err.println("### Exception caught in run method of MultiServerThread : "  + oe.getMessage());
			oe.printStackTrace();
		}		
  } // end of run()

	/**
	 Asking for information
	*/
	
	private String doAsk(Vector parameters) {
		
		// get the ask command
		String command = (String)parameters.get(0);
			
		if (command.equalsIgnoreCase("primary")) {
			return status.getPrimaryServer();
		}
		
		else if (command.equalsIgnoreCase("isAlive")) {
			return "*";
		}

		else if (command.equalsIgnoreCase("info")) {
			return "elipva Xander Notification Server version 1.0 (beta1).";
		}

			
		else if (command.equalsIgnoreCase("listall")) {
			return status.getServersAsString();	
		}
		
		else if (command.equalsIgnoreCase("port")) {
			try {
				String portnum = (String)parameters.get(1);
				return Integer.toString(status.getPort(portnum));
			}
			catch (ArrayIndexOutOfBoundsException e) {
				return "##Error: You must enter the IP address of the server.";
			}			
		}		
		
		else {
			return "##Error: No such ask command.";
		}		
	} // end of doAsk

	/**
	 Request for change
	*/
	
	private boolean doChange(Vector parameters) {
		
		String command = (String)parameters.get(0);
		
		//System.out.println("Parameter1: " + parameter1);		

		if (command.equalsIgnoreCase("add")) {
			try {
				String hostname = (String)parameters.get(1);
				String portnum = (String)parameters.get(2);
				
				int port = Integer.parseInt(portnum);
				status.addServer(hostname, port);
				return true;
			}
			catch (ArrayIndexOutOfBoundsException e) {
				return false;
			}
			
		}
		
		else if (command.equalsIgnoreCase("remove")) {
			try {
				String hostname = (String)parameters.get(1);
				
				status.removeServer(hostname);
				return true;
			}
			catch (ArrayIndexOutOfBoundsException e) {
				return false;			}
			
		}
		
		else {
			return false;
		}	
	} // end of doChange()

	
} // end of MultiServerThread class