package mercury;

import java.util.*;

public class Task implements java.io.Serializable {

	private String taskid = new String();
	private String taskName = new String();
	private String application = new String();
	private ArrayList actions = new ArrayList();
	private ArrayList conditions = new ArrayList();
	private ArrayList parameters = new ArrayList();
	
	public Task() {
		init();
	}
	
	public Task(String id) {
		taskid = id;
	}

	public String getID() {
		return taskid;	
	}

	public void setID (String id)  {
		taskid = id;
	}

	public String getName() {
		return taskName;
	}
	
	public void setName(String name) {
		taskName = name;
	}
	
	public void setApplication (String app) {
		application = app;
	}
	
	public String getApplication() {
		return application;
	}

	public void addCondition(Condition cond) {
		conditions.add(cond);
	}

	public void addAction(Action act) {
		actions.add(act);
	}
	
	public void execute() {
		for (Iterator i = actions.iterator(); i.hasNext();) {
			Action act = (Action)i.next();
			act.execute(parameters);
		}
	}
	
	public boolean checkAllConditions() {
		for (Iterator i = conditions.iterator(); i.hasNext();) {
			Condition cond = (Condition)i.next();
			boolean passed = cond.check();
			if (!passed) {
				return false;				
			}
			else {
				parameters.add(cond.getResult());				
			}
		}
		return true;
	}

	public Condition [] getAllConditions() {
		Condition [] conds = new Condition[conditions.size()];
		int count = 0;
		for (Iterator i=conditions.iterator();i.hasNext();) {
			conds[count] = (Condition)i.next();	
		}
		return conds;
	}


	private void init() {
		IDGenerator gen = new IDGenerator();
		taskid = gen.generateID();					
	}
	
	public boolean equals(Object o) {
		if (o instanceof Task) {
			Task t = (Task)o;
			return (t.getID().equals(taskid));
		}
		else {
			throw new ClassCastException("Can only check with another Task object.");
		}		
	}

	/**
	Duplicates this task -- return a new task with the same task ID and same name
	and same actions. The new task does not have any conditions.
	*/	
	public Task duplicate() {
		Task task = new Task(taskid);	
		task.setName(taskName);
		for (Iterator i=actions.iterator(); i.hasNext();) {
			task.addAction((Action)i.next());	
		}
		
		return task;
	}
		

}
