package mercury;

import java.util.*;
import java.io.*;
import mercury.exception.*;
import mercury.util.*;

public class TaskProcessor {

	Config config = Config.getInstance();
	boolean debug = new Boolean(config.getProperty("debug")).booleanValue();
	
	public void process(Task task) {
	
		TaskRegistry registry = new TaskRegistry();
	
		try {
			if (debug) System.out.println("TaskProcessor processing task:" + task.getID());
			if (task.checkAllConditions()) {
				task.execute();
				
				/*
				Remove the task from the registry
				*/
				registry.removeTask(task);
				
				Task newTask = task.duplicate();
				boolean toRepeat = false;
				
				Condition [] conditions = task.getAllConditions();
				
				if (debug) System.out.println("No of conditions found:" + conditions.length);
				
				for (int i=0; i < conditions.length; i++) {				
					if (conditions[i] instanceof TimeCondition){
						if (debug) System.out.println("Found a time condition!");
						
						TimeCondition timeCondition = (TimeCondition)conditions[i];
						
						if (debug) System.out.println("Time condition:" + timeCondition.getTime().toString());
						
						if (timeCondition.isRepeated()) {
							if (debug) System.out.println("Time condition is repeated! Adding a new time condition!");
							toRepeat = true;
							TimeCondition newTimeCondition = new TimeCondition();
							int freq = timeCondition.getFrequency();
							
							
							int multiplier = timeCondition.getMultiplier();
							Date time = timeCondition.getTime();
							
							GregorianCalendar g = new GregorianCalendar();
							g.setTime(time);

							/*
							If the time condition is to be repeated every time the task is processed, let the same time
							be set. Otherwise, roll to the new time
							*/
							if (freq != TimeCondition.EVERYTIME) {
								while(g.before(Calendar.getInstance())) {
									g.roll(freq,multiplier);
								}
							}
							
							if (debug) System.out.println("Old condition time:" + time.toString());
							
							newTimeCondition.setTime(g.getTime());
							newTimeCondition.setFrequency(freq);
							newTimeCondition.setMultiplier(multiplier);
							
							if (debug) System.out.println("New condition time:" + newTimeCondition.getTime().toString());
							newTask.addCondition(newTimeCondition);
						}
					}
					else {
						newTask.addCondition(conditions[i]);
					}
					
				}
				if (toRepeat) {
					if (debug) System.out.println("Registered new task!");
					registry.registerTask(newTask);	
				}
			}
		}
		catch (Exception e) {
			System.err.println("## Exception in processing tasks.");	
			e.printStackTrace();
		}
				
	}

}
