package mercury;

import java.util.*;
import java.io.*;
import java.sql.*;
import javax.sql.*;
import mercury.exception.*;
import mercury.util.*;

public class TaskRegistry implements java.io.Serializable {

	HashMap registry = new HashMap();
	Config config = Config.getInstance();
	boolean debug = new Boolean(config.getProperty("debug")).booleanValue();
	
			
	public void registerTask(Task task) throws NullPointerException {
		System.out.println("registering task:" + task.getID());
		if (task != null) {		
			if (task.getAllConditions() == null) {
				throw new NullPointerException("### Error : Every task must have at least 1 condition.");			
			}
					
			DBConnectionManager mgr = DBConnectionManager.getInstance();
			
			if (mgr == null)
				System.err.println("## no connection manager found");
			
			Connection conn = mgr.getConnection(Config.getInstance().getProperty("db.poolname"));
			if (debug) System.out.println("Pool name:"  +Config.getInstance().getProperty("db.poolname"));
			if (conn == null)
				System.err.println("## no connection found");
	
			try {
				conn.setAutoCommit(false);
				ByteArrayOutputStream ba_out = new ByteArrayOutputStream();
				ObjectOutputStream obj_out = new ObjectOutputStream(ba_out);
				obj_out.writeObject(task);
	
				byte [] data = ba_out.toByteArray();
				PreparedStatement ps = conn.prepareStatement("INSERT INTO task_registry (task_id,task_data) VALUES ('" + task.getID() + "' , ?)");
				ps.setBytes(1, data);
				ps.executeUpdate();
	
				ba_out.close();
				obj_out.close();
				ps.close();
				
				conn.commit();
				
				/* Then write to local cache */
				registry.put(task.getID(), task);
			}
			catch (Exception e) {
				e.printStackTrace();
			}		
			finally {
				mgr.freeConnection(Config.getInstance().getProperty("db.poolname"),conn);
			}		
		}
		else {
			throw new NullPointerException("### Error : You tried to add a null task.");			
		}
		
	}
	
	public void removeTask(Task task) throws TaskNotFoundException {
		if (task != null) {
			
			Object obj = registry.remove(task.getID());
			DBConnectionManager mgr = DBConnectionManager.getInstance();
			Connection conn = mgr.getConnection(Config.getInstance().getProperty("db.poolname"));
			try {
				Statement stmt = conn.createStatement();
				
				/* oracle code */
				stmt.executeQuery("DELETE FROM task_registry WHERE task_id = '" + task.getID() + "'");
				stmt.close();
				
			}
			catch (java.sql.SQLException e) {
				e.printStackTrace();	
			}
			finally {
				mgr.freeConnection(Config.getInstance().getProperty("db.poolname"),conn);
			}
		}
		else {
			throw new NullPointerException("### Error : You tried to remove a null task.");						
		}
	}
	
	public void removeTask(String taskID) throws TaskNotFoundException {
		if (taskID != null) {
			removeTask(new Task(taskID));
		}
		else {
			throw new NullPointerException("### Error : You tried to remove a null task.");						
		}
	}
	
	public Task getTask(String taskID)  throws TaskNotFoundException  {

		Task task = (Task)registry.get(taskID);
		if (task != null || registry.containsKey(taskID))
			return task;
		
		DBConnectionManager mgr = DBConnectionManager.getInstance();
		Connection conn = mgr.getConnection(Config.getInstance().getProperty("db.poolname"));
		
		try {
			if (task == null || !registry.containsKey(taskID)) {
				Statement stmt = conn.createStatement();
				ResultSet resSet = stmt.executeQuery("SELECT task_data FROM task_registry WHERE task_id = '" + taskID + "'");
				if (resSet.next()) {
					Blob data = resSet.getBlob(1);	
					InputStream in = data.getBinaryStream();
					ObjectInputStream obj_in = new ObjectInputStream(in);
					task = (Task)obj_in.readObject();	
					in.close();
				}
				else {
					stmt.close();					
					return null;
				}
				stmt.close();
			}
		}
		catch (java.sql.SQLException e) {
			e.printStackTrace();	
		}
		catch (java.io.IOException e) {
			e.printStackTrace();	
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();	
		}
		catch (Exception e) {
			e.printStackTrace();	
			
		}
		
		finally {
			mgr.freeConnection(Config.getInstance().getProperty("db.poolname"),conn);
		}
		return task;
		
	}
	
	public Task [] getTasks(String taskName) throws TaskNotFoundException  {
		ArrayList tasks = new ArrayList();
		
		Task [] allTasks = getAllTasks();
		
		for (int i = 0; i < allTasks.length; i++) {
			Task t = allTasks[i];
			if (t.getName().equals(taskName)) {
				tasks.add(t);	
			}
		}
		
		if (tasks.isEmpty()) {
			throw new TaskNotFoundException("### Error : No such tasks in the task registry.");		
		}
		

		Task [] taskList = new Task[tasks.size()];
		int count = 0;
		for (Iterator j = tasks.iterator(); j.hasNext(); ) {
			taskList[count] = (Task)j.next();
			count++;
		}
		return taskList;

			
	}
	
	public Task [] getAllTasks() throws TaskNotFoundException {

		registry.clear();
		DBConnectionManager mgr = DBConnectionManager.getInstance();
		Connection conn = mgr.getConnection(Config.getInstance().getProperty("db.poolname"));
		
		try {
			Statement stmt = conn.createStatement();
	
			ResultSet resSet = stmt.executeQuery("SELECT task_data FROM task_registry");
			while (resSet.next()) {
				Blob data = resSet.getBlob(1);	
				InputStream in = data.getBinaryStream();
				ObjectInputStream obj_in = new ObjectInputStream(in);
				Task task = (Task)obj_in.readObject();	
				registry.put(task.getID(), task);
				in.close();
			}
			stmt.close();
		}
		catch (java.sql.SQLException e) {
			e.printStackTrace();	
		}
		catch (java.io.IOException e) {
			e.printStackTrace();	
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();	
		}		
		finally {
			mgr.freeConnection(Config.getInstance().getProperty("db.poolname"),conn);
		}
		
		/* -- get tasks from the cache --*/
		
		Collection c = registry.values();
		Iterator i = c.iterator();
		Task [] taskList = new Task [c.size()];
		
		int count = 0;
		while(i.hasNext() || count < c.size()) {
			taskList[count] = (Task)i.next();
			count++;
		}		
		return taskList;
	
	}


	public boolean hasTask(String taskID)   {

		Task task = (Task)registry.get(taskID);
		if (task != null || registry.containsKey(taskID)) {
			return true;
		}
		
		DBConnectionManager mgr = DBConnectionManager.getInstance();
		Connection conn = mgr.getConnection(Config.getInstance().getProperty("db.poolname"));
		
		try {
			if (task == null || !registry.containsKey(taskID)) {
				Statement stmt = conn.createStatement();
				ResultSet resSet = stmt.executeQuery("SELECT task_data FROM task_registry WHERE task_id = '" + taskID + "'");
				if (resSet.next()) {
					stmt.close();
					return true;
				}
				else {
					stmt.close();
					return false;
				}
			}
		}
		catch (java.sql.SQLException e) {
			e.printStackTrace();	
		}
		catch (Exception e) {
			e.printStackTrace();	
			
		}
		finally {
			mgr.freeConnection(Config.getInstance().getProperty("db.poolname"),conn);
		}
		return false;
	}


}
